package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.model.AlarmPatchParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import com.megatron.common.model.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 告警
 *
 * @author zhangyf
 * @since 2019-02-18
 */
@Tag(name = "告警")
@RestController
@RequestMapping("/v1/alarms")
public class AlarmController {

    private final String CONFIRM = "confirm";
    private final String SOLVE = "solve";

    @Autowired
    private AlarmService alarmService;

    /**
     * 告警列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @return 告警列表
     */
    @GetMapping
    @Operation(summary = "告警列表查询")
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return new GeneralResult<>(true, alarmService.list(pager, context), "查询成功");
    }


    /**
     * 用户首页显示
     *
     * @param context 用户上下文
     * @return 首页告警列表
     */
    @GetMapping("/portal")
    @Operation(summary = "首页告警列表查询")
    public GeneralResult portal(@Value(Common.REQ_CONTEXT) RequestContext context) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("eventAlarm", 0), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("status", "SOLVED"), Sign.UEQ));
        Pager pager = new Pager(1, 10, params, null);
        return new GeneralResult<>(true, alarmService.list(pager, context), "查询成功");
    }

    /**
     * 详情查询
     *
     * @param id 告警id
     * @return 告警对象详情
     */
    @Operation(summary = "告警详情查询")
    @GetMapping("/{id}")
    public GeneralResult<Alarm> detail(@PathVariable(value = Common.ID) Long id) {
        return alarmService.detail(id);
    }


    /**
     * 告警确认、告警解决
     *
     * @param param   修改参数
     * @param context 当前用户上下文
     * @return 解决成功与否
     */
    @PatchMapping
    @Operation(summary = "告警解决/告警确认")
    public GeneralResult patch(@Value(Common.REQ_CONTEXT) RequestContext context,
                               @RequestBody AlarmPatchParam param) {
        if (CONFIRM.equals(param.getAction())) {
            try {
                return alarmService.confirm(param.getIds(), context);
            } catch (RuntimeException e) {
                return new GeneralResult<>(false, "确认告警失败：" + e.getMessage());
            }
        }
        if (SOLVE.equals(param.getAction())) {
            try {
                alarmService.solve(param.getIds(), context, param.getDealResult());
                return new GeneralResult<>(true, "告警已解决");
            } catch (RuntimeException e) {
                return new GeneralResult<>(false, "解决告警失败：" + e.getMessage());
            }
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 告警统计
     *
     * @param paramList 统计参数
     * @param context   当前用户上下文
     * @return 告警统计结果
     */
    @Operation(summary = "告警统计")
    @GetMapping(value = "/stats")
    public GeneralResult stats(@RequestBody List<Param> paramList,
                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != paramList) {
            return alarmService.stats(paramList, context);
        } else {
            return new GeneralResult<>(false, "参数格式不正确");
        }
    }

    /**
     * 告警统计数据饼状图、告警数量柱状堆积图
     *
     * @param vendorId   平台id
     * @param target     标签
     * @param type       类型
     * @param vendorType 平台类型
     * @param start      开始时间
     * @param end        结束时间
     * @param action     图形类型展示
     * @param context    当前用户上下文
     * @return 渲染数据
     */
    @GetMapping("/chart")
    @Operation(summary = "告警统计数据饼状图、告警数量柱状堆积图")
    public GeneralResult alarmLevel(@RequestParam(value = "vendorId", required = false) Long vendorId,
                                    @RequestParam(value = "target", required = false) Object target,
                                    @RequestParam(value = "type", required = false) String type,
                                    @RequestParam(value = "vendorType", required = false) String vendorType,
                                    @RequestParam(value = "start", required = false) Long start,
                                    @RequestParam(value = "end", required = false) Long end,
                                    @RequestParam(value = Common.ACTION, required = false) String action,
                                    @RequestParam(value = "time", required = false) String time,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (action) {
            //pie chart - 饼状图
            case "pieChart":
                return this.alarmService.alarmLevel(context, start, end, vendorId, target, type, vendorType, time);
            //bar chart - 柱状图
            case "barChart":
                return alarmService.alarmStats(context, type, vendorId, target, start, end, vendorType);
            case "vendorChart":
                return alarmService.vendorAlarmStats(context, start, end, vendorType);
            default:
                return new GeneralResult<>(false, "参数格式不正确");
        }
    }
}
