package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.intf.RuleGroupService;
import com.bocloud.cms.model.AlarmBinding;
import com.bocloud.cms.model.DeleteIdList;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author fanzx
 * @date 2021/09/18
 */
@RestController
@Tag(name = "告警规则")
@RequestMapping("/v1/rulegroups")
public class RuleGroupController {

    @Autowired
    private RuleGroupService groupService;

    /**
     * 新增告警规则功能
     *
     * @param ruleGroup 告警规则对象
     * @param context   当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @PostMapping
    @Operation(summary = "告警规则添加")
    public GeneralResult create(@Value(Common.REQ_CONTEXT) RequestContext context,
                                @RequestBody RuleGroup ruleGroup) {
        return groupService.create(ruleGroup, context);
    }

    /**
     * 告警规则分页列表展示
     *
     * @param pager   分页   模糊搜索params
     * @param context 当前用户上下文
     * @return 告警规则列表
     */
    @GetMapping
    @Operation(summary = "告警规则列表")
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.list(pager, context);
    }

    /**
     * 删除告警规则（单条）
     *
     * @param id      告警规则id
     * @param context 当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除告警规则")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.remove(id, context);
    }

    /**
     * 批量删除告警规则
     *
     * @param param   告警规则列表
     * @param context 当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @DeleteMapping
    @Operation(summary = "批量删除告警规则")
    public GeneralResult batchRemove(@RequestBody DeleteIdList param,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.batchRemove(param.getIds(), context.getTarget());
    }

    /**
     * 修改告警规则
     *
     * @param id        告警规则id
     * @param ruleGroup 修改后的告警规则
     * @param context   当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改告警规则")
    public GeneralResult modify(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context,
                                @RequestBody RuleGroup ruleGroup) {
        return groupService.modify(id, ruleGroup, context);
    }

    /**
     * 告警规则详情展示
     *
     * @param id 告警规则id
     * @return 告警规则详情对象
     */
    @GetMapping("/{id}")
    @Operation(summary = "告警规则详情")
    public GeneralResult<RuleGroup> detail(@PathVariable(value = Common.ID) Long id) {
        try {
            RuleGroup ruleGroup = groupService.detail(id);
            return new GeneralResult<>(true, ruleGroup, "告警规则详情展示成功");
        } catch (RuntimeException e) {
            return new GeneralResult<>(false, "告警规则详情展示失败：" + e.getMessage());
        }
    }

    /**
     * 开启/关闭 告警规则
     *
     * @param ruleGroup 告警规则对象
     * @param context   当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @PatchMapping("/enable")
    @Operation(summary = "告警规则状态")
    public GeneralResult enabled(@RequestBody RuleGroup ruleGroup,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.enabled(ruleGroup.getId(), ruleGroup.getEnable(), context);
    }

    /**
     * 云平台关联告警规则功能
     *
     * @param alarmBinding 告警绑定帮助类
     * @param context      当前用户上下文      上下文对象
     * @return 布尔值 成功或者失败
     */
    @PatchMapping("/binding")
    @Operation(summary = "关联告警规则")
    public GeneralResult binding(@RequestBody AlarmBinding alarmBinding,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.binding(alarmBinding.getRuleGroupIds(), alarmBinding.getResourceId(), alarmBinding.getResourceType(), context);
    }

    /**
     * 云平台关联告警规则功能 获取规则资源绑定列表
     * 通过resourceId,查询到已经关联的告警规则列表
     *
     * @param resourceId 资源id
     * @param context    当前用户上下文    上下文对象
     * @return 布尔值 成功或者失败
     */
    @GetMapping("/{id}/resources")
    @Operation(summary = "查询已关联的告警规则")
    public GeneralResult bindingList(@PathVariable("id") String resourceId,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return groupService.bindingList(resourceId, context);
    }
}