package com.bocloud.cms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.MonitorAllocationRatio;
import com.bocloud.cms.entity.MonitorAuth;
import com.bocloud.cms.intf.AllocationRatioService;
import com.bocloud.cms.intf.AuthService;
import com.bocloud.cms.intf.QueryService;
import com.bocloud.cms.intf.VendorService;
import com.bocloud.cms.model.VendorActionParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 平台监控
 *
 * @author 王文凯
 * @since 2019-2-15
 */
@Tag(name = "监控平台")
@RestController
@RequestMapping("/v1/vendors")
public class VendorController {

    @Autowired
    private VendorService vendorService;
    @Autowired
    private AllocationRatioService allocationRatioService;
    @Autowired
    private AuthService authService;
    @Autowired
    private QueryService queryService;

    /**
     * 监控openstack平台概览
     *
     * @param id 平台id
     * @return 概览数据展示
     */
    @GetMapping
    @Operation(summary = "OpenStack平台监控概览")
    public GeneralResult vendorOverview(@RequestParam(value = "id", required = false) Long id) {
        return queryService.vendor(id);
    }

    /**
     * 开启/关闭监控
     *
     * @param param   参数控制
     * @param context 当前用户上下文
     * @return 成功/失败
     */
    @PatchMapping
    @Operation(summary = "开启/关闭平台监控")
    public GeneralResult open(@RequestBody VendorActionParam param,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        if ("open".equals(param.getAction())) {
            return vendorService.open(context, param.getIds());
        }
        if ("close".equals(param.getAction())) {
            return vendorService.close(context, param.getIds());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 配置超分比
     *
     * @param allocationRatio 百分比配置参数
     * @return 成功/失败
     */
    @PostMapping("/ratio")
    @Operation(summary = "配置OpenStack超分比")
    public GeneralResult saveRatio(@RequestBody MonitorAllocationRatio allocationRatio) {
        return allocationRatioService.save(allocationRatio);
    }

    /**
     * 查询超分比详情
     *
     * @param vendorId 平台id
     * @return 百分比数据展示
     */
    @GetMapping("/ratio")
    @Operation(summary = "查询OpenStack超分比")
    public GeneralResult queryRatio(Long vendorId) {
        return allocationRatioService.query(vendorId);
    }

    /**
     * 创建云平台监控运维认证信息
     *
     * @param id          平台id
     * @param monitorAuth 监控指标认证信息
     * @param context     当前用户上下文
     * @return 成功/失败
     */
    @PostMapping("/{id}/authentications")
    @Operation(summary = "华为云运维认证信息")
    public GeneralResult saveAuthInfo(@PathVariable(value = Common.ID) Long id,
                                      @RequestBody MonitorAuth monitorAuth,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return authService.save(monitorAuth, context, id);
    }

    /**
     * 详情
     *
     * @param id 认证平台id
     * @return 查询认证输入信息
     */
    @GetMapping("/{id}/authentications")
    @Operation(summary = "华为云运维认证信息")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return new GeneralResult<>(true, authService.detail(id), "查询成功");
    }

    /**
     * 平台信息下载
     *
     * @param id   平台id
     * @param uuid 唯一标识
     * @return 文件下载
     */
    @GetMapping("/download")
    @Operation(summary = "平台信息下载")
    public GeneralResult download(Long id, String uuid) {
        return queryService.download(id, uuid);
    }

    /**
     * 配置xsky prometheus信息
     *
     * @param params 配置参数
     * @return 成功/失败
     */
    @PostMapping("/{id}/monitor")
    @Operation(summary = "配置xsky prometheus信息")
    public GeneralResult updateXskyMonitoringInfo(@PathVariable(value = Common.ID) Long id, @RequestParam(value = Common.PARAMS, required = false) String params) {
        JSONObject object = JSONObject.parseObject(params);
        if (null != object) {
            vendorService.updateXskyMonitoringInfo(id, object);
            return new GeneralResult<>(true, "success");
        }
        return ResultTools.formatErrResult();
    }


    /**
     * 查询所有FC平台
     *
     * @return FC平台列表
     */
    @GetMapping("/authentications")
    @Operation(summary = "查询所有FC平台")
    public GeneralResult listFC() throws Exception {
        return new GeneralResult<>(true, authService.listFC(), "查询成功");
    }
}

