package com.bocloud.cms.repository;

import com.bocloud.cms.entity.Metric;
import com.google.common.collect.Lists;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 监控指标
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2021/09/18
 */
@Repository
public class MetricRepository extends BasicGenericDao<Metric, Long> {

    public MetricRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 查询监控指标列表 获取到的指标为开启状态
     *
     * @param resourceType 资源类型
     * @return 监控指标集合
     */
    public List<Metric> listMetrics(String resourceType) {
        String sql = "select * from monitor_metric where active = 1 and status = 1 ";
        Map<String, Object> paramMap = null;
        if (StringUtils.isNotBlank(resourceType)) {
            sql += " and resource_type = :resourceType ";
            paramMap = MapTools.simpleMap("resourceType", resourceType);
        }
        sql += " order by tag,id ";
        List<Metric> metrics = this.list(Metric.class, sql, paramMap);
        List<Metric> resultList = Lists.newArrayList();
        for (Metric metric : metrics) {
            resultList.add(metric);
        }
        return resultList;
    }


    /**
     * 有效监控指标列表
     *
     * @return 监控指标列表
     */
    public List<Metric> list() {
        return listMetrics(null);
    }

    /**
     * 根据metricId查询Metric信息
     *
     * @param metricId 监控指标ID
     * @return 监控指标
     */
    public Metric getByMetricId(Long metricId) {
        String sql = "select * from monitor_metric where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", metricId);
        List<Metric> metrics = this.list(Metric.class, sql, paramMap);
        return ListTool.isEmpty(metrics) ? null : metrics.get(0);
    }

    /**
     * 监控资源配置功能 解绑操作
     * 通过资源类型绑定监控指标展示状态
     *
     * @param resourceType 资源类型
     */
    public void unBindingMetrics(String resourceType) {
        String sql = "UPDATE monitor_metric SET status = 0 WHERE active = 1  AND resource_type = :resourceType";
        Map<String, Object> params = MapTools.simpleMap("resourceType", resourceType);
        this.execute(sql, params);
    }

    /**
     * 监控资源配置功能 根据id进行绑定操作
     * 通过resourceType 绑定监控指标展示状态
     *
     * @param resourceType 资源类型
     */
    public void bindingMetrics(List<Integer> metricsIds, String resourceType) {
        String sql = "UPDATE monitor_metric SET status = 1 WHERE active = 1  AND resource_type = :resourceType AND id in (:metricsIds)";
        Map<String, Object> params = MapTools.simpleMap("resourceType", resourceType);
        params.put("metricsIds", metricsIds);
        this.execute(sql, params);
    }

    /**
     * 告警配置关联列表中展示metric指标列表数量
     *
     * @param params 参数集合
     * @return 数量
     */
    public int metricCount(List<Param> params) {
        String sql = "select count(1) from monitor_metric where active = 1 ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 告警配置关联列表中展示metric指标列表
     *
     * @param pager 分页
     * @return 监控指标列表
     */
    public List<Metric> metricList(Pager pager) {
        String sql = "select mm.*, mrt.name resource_name from monitor_metric mm left join monitor_resource_type mrt on  mm.resource_type = mrt.resource_type where mm.active = 1\n";
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        sql = getQueryBuilder().buildRaw(sql, pager, "mm");
        return this.list(Metric.class, sql, paramMap);
    }

    /**
     * 逻辑删除监控指标
     *
     * @param id 监控指标ID
     */
    public void deletedMetric(Long id) {
        String sql = "update monitor_metric set active = 0,status = 0 where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        this.execute(sql, paramMap);
    }

    /**
     * 获取该资源类型下所有status为开启的指标列表
     *
     * @param resourceType 资源类型
     * @return 监控指标集合
     */
    public List<Metric> getByResourceType(String resourceType) {
        String sql = "SELECT * FROM monitor_metric WHERE active = 1 AND status = 1 AND resource_type = :resourceType";
        Map<String, Object> params = MapTools.simpleMap("resourceType", resourceType);
        return this.list(Metric.class, sql, params);
    }
}
