package com.bocloud.cms.repository;

import com.bocloud.cms.entity.RuleGroup;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 告警策略
 *
 * @author fanzx
 * @date 2021/09/18
 */
@Repository
public class RuleGroupRepository extends BasicGenericDao<RuleGroup, Long> {

    public RuleGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据policyId去查询总数
     *
     * @param id      ID
     * @param context 当前用户上下文
     * @return 数量
     */
    public Long countByPolicy(Long id, RequestContext context) {
        String sql = "select count(*) from monitor_rule_group a where a.is_deleted = 0 and a.policy_id = :policyId and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("policyId", id);
        paramMap.put("tenantId", context.getTenant());
        return this.countQuery(sql, paramMap);
    }

    /**
     * 根据name查询RuleGroup对象
     *
     * @param id      ID
     * @param name    名称
     * @param context 当前用户上下文
     * @return 告警策略
     */
    public RuleGroup getByName(Long id, String name, RequestContext context) {
        String sql = "select * from monitor_rule_group where is_deleted = 0 and name = :name and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("tenantId", context.getTenant());
        if (null != id) {
            sql += "and id = :id ";
            paramMap.put("id", id);
        }
        List<RuleGroup> ruleGroups = this.list(RuleGroup.class, sql, paramMap);
        return ListTool.isEmpty(ruleGroups) ? null : ruleGroups.get(0);
    }

    /**
     * 查询告警策略总数
     *
     * @param params  参数集合
     * @param context 当前用户上下文
     * @return 总数
     */
    public int count(List<Param> params, RequestContext context) {
        String sql = "select count(1) from monitor_rule_group where is_deleted = 0 and tenant_id = :tenantId  ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        paramMap.put("tenantId", context.getTenant());
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }


    /**
     * 查询告警策略列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 告警策略列表
     */
    public List<RuleGroup> list(Pager pager, RequestContext context) {
        String sql = "select g.*,p.name as policy_name from monitor_rule_group g " +
                " left join monitor_alarm_send_policy p on g.policy_id = p.id " +
                " where g.is_deleted = 0 and g.tenant_id = :tenantId ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and g.creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "g");
        return this.list(RuleGroup.class, sql, paramMap);
    }

    /**
     * 根据ruleGroupId"删除"表信息
     *
     * @param id ID
     * @return 是否成功
     */
    public boolean delete(Long id) {
        String sql = "update monitor_rule_group set is_deleted = 1, gmt_modify = :gmtModify where id = :groupId";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 判断告警是否存在
     *
     * @param name    创建/修改时传入的告警名称
     * @param context 当前用户上下文 上下文对象
     * @return 是否存在
     */
    public boolean existsSameName(String name, RequestContext context) {
        String sql = "select count(1) from monitor_rule_group where name = :name and tenant_id = :tenantId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("tenantId", context.getTenant());
        if (0L != context.getTenant()) {
            sql += " and creator_id = :creatorId ";
            paramMap.put("creatorId", context.getTarget());
        }
        return this.countQuery(sql, paramMap).intValue() > 0;
    }
}
