package com.bocloud.cms.service;

import com.bocloud.cms.entity.AlarmTemplate;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.intf.AlarmTemplateService;
import com.bocloud.cms.repository.AlarmTemplateRepository;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 告警模板
 *
 * @author fanzx
 * @version 5.5.0
 * @date 2022/2/21 18:01
 */
@Service
@Slf4j
public class AlarmTemplateServiceImpl implements AlarmTemplateService {


    @Autowired
    private AlarmTemplateRepository alarmTemplateRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public GeneralResult list(Pager pager, RequestContext context) {
        GridBean gridBean;
        try {
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = this.alarmTemplateRepository.count(pager.getParams(), context);
            List<AlarmTemplate> alarmTemplates = this.alarmTemplateRepository.list(pager, context);
            alarmTemplates.forEach(alarmTemplate -> {
                if (null != alarmTemplate.getCreatorId()) {
                    String name = redisTemplate.opsForValue().get("User_Name_" + alarmTemplate.getCreatorId());
                    alarmTemplate.setCreateName(name);
                }
            });
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, alarmTemplates);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("query template list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult create(AlarmTemplate alarmTemplate, RequestContext context) {

        try {
            Assert.notNull(alarmTemplate, "未获取到任何数据!");
            alarmTemplate.setCreatorId(context.getTarget());
            alarmTemplate.setTenantId(context.getTenant());
            alarmTemplate.setGmtCreate(new Date());
            alarmTemplate.setIsDeleted(false);
            this.alarmTemplateRepository.save(alarmTemplate);
            return new GeneralResult<>(true, "新增成功");
        } catch (Exception e) {
            log.error("create templates fail:", e);
            return new GeneralResult<>(false, "新增失败");
        }
    }

    @Override
    public GeneralResult modify(AlarmTemplate alarmTemplate, RequestContext context) {
        try {
            AlarmTemplate template = this.alarmTemplateRepository.get(AlarmTemplate.class, alarmTemplate.getId());
            Assert.notNull(template, "未查询到任何数据!");
            alarmTemplate.setCreatorId(template.getCreatorId());
            alarmTemplate.setMenderId(context.getTarget());
            alarmTemplate.setTenantId(context.getTenant());
            alarmTemplate.setIsDeleted(false);
            this.alarmTemplateRepository.update(alarmTemplate);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("modify templates fail:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult remove(Long id, RequestContext context) {
        String path = RuleGroup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            org.springframework.util.Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            AlarmTemplate alarmTemplate = this.alarmTemplateRepository.get(AlarmTemplate.class, id);
            Assert.notNull(alarmTemplate, " 数据不存在");
            alarmTemplate.setIsDeleted(true);
            this.alarmTemplateRepository.update(alarmTemplate);
            return new GeneralResult<>(true, "告警模板删除成功!");

        } catch (Exception e) {
            log.error("remove template error:", e);
            return new GeneralResult<>(false, "告警模板删除失败：" + e.getMessage());
        }
    }

    @Override
    public GeneralResult detail(Long id) {
        try {
            AlarmTemplate alarmTemplate = this.alarmTemplateRepository.get(AlarmTemplate.class, id);
            if (alarmTemplate.getIsDeleted()) {
                return new GeneralResult<>(false, new AlarmTemplate(), "查询失败,数据不存在");
            }
            Assert.notNull(alarmTemplate, "未查询到任何数据!");
            return new GeneralResult<>(true, alarmTemplate, "查询成功");
        } catch (Exception e) {
            log.error("Query alarm template detail error:", e);
            return new GeneralResult<>(false, new AlarmTemplate(), "查询失败" + e.getMessage());
        }
    }

    @Override
    public GeneralResult batchRemove(List<Long> ids) {
        int rightCount = 0;
        int wrongCount = 0;
        try {
            Assert.notEmpty(ids, " 至少选择一个告警模板删除");
            for (Long id : ids) {
                AlarmTemplate alarmTemplate = alarmTemplateRepository.get(AlarmTemplate.class, id);
                alarmTemplate.setIsDeleted(true);
                this.alarmTemplateRepository.update(alarmTemplate);
                rightCount++;
            }
        } catch (Exception e) {
            log.error("alarm template batch delete error!", e);
            return new GeneralResult<>(false, "批量删除告警模板失败:" + e.getMessage());
        }
        return new GeneralResult<>(true, "成功删除【" + rightCount + "】个，失败【" + wrongCount + "】个");
    }
}
