package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.ApsaraStackMysqlService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.RdsMysqlModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/9/29 15:46
 */
@Slf4j
@Service
public class ApsaraStackMysqlServiceImpl implements ApsaraStackMysqlService, InitializingBean {

    @Autowired
    private CmpCloudServerService cmpCloudServerService;
    @Autowired
    private AlarmService alarmService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;


    @Override
    public GeneralResult list(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult result = cmpCloudServerService.listMysql(pager, request.getHeader(Common.TOKEN));
        if (result.isFailed()) {
            return new GeneralResult(false,"查询数据库列表失败，请检查服务状态");
        }
        JSONObject data = (JSONObject) result.getData();
        List<RdsMysqlModel> list = data.getJSONArray(Common.ROWS).toJavaList(RdsMysqlModel.class);
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_APSARASTACK_RDS_MYSQL.name(), targets, context);
        for (RdsMysqlModel rdsModel : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .anyMatch(a -> a.getResourceId().equals(rdsModel.getId().toString()));
            rdsModel.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }


    @Override
    public List<RdsMysqlModel> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpCloudServerService.listMysql(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(RdsMysqlModel.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(RdsMysqlModel targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>();
        params.put("instanceId", targetObject.getUuid());
        // 数据库类型
        params.put("type", "mysql");
        return params;
    }

    @Override
    public RdsMysqlModel parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String uuid = String.valueOf(alert.getLabels().get("instanceId"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
        GeneralResult result = cmpCloudServerService.listMysql(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<RdsMysqlModel> list = data.getJSONArray(Common.ROWS).toJavaList(RdsMysqlModel.class);
        Assert.isTrue(list != null && !list.isEmpty(), "apsarastack mysql not found,uuid:" + uuid);
        return list.get(0);
    }

    @Override
    public void destroy(RdsMysqlModel object, String catalog) throws Exception {
        log.info("destroy apsarastack Mysql  : " + object.getId());
        alarmService.solveByCloudServer(object.getId(), catalog, 1L, "系统解决");
    }


    @Override
    public void wrapAlarm(Alarm alarm, RdsMysqlModel object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getName();
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, RdsMysqlModel object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_APSARASTACK_RDS_MYSQL, this);
    }
}
