package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.ResourceCategory;
import com.bocloud.cms.intf.*;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.model.RdsModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.mms.MmsRdsService;
import com.bocloud.pms.entity.Rds;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * RdsService实现类
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class RdsServiceImpl implements RdsService, InitializingBean {


    @Autowired
    private MmsRdsService mmsRdsService;
    @Autowired
    private StorageConfig storageConfig;
    @Autowired
    private PrometheusService prometheusService;
    @Autowired
    private PrometheusRuleService prometheusRuleService;
    @Autowired
    private TelegrafService telegrafService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public void init(Rds rds) throws Exception {
        switch (ResourceCategory.valueOf(rds.getType())) {
            case MySQL:
                generateMySqlConf(rds);
                break;
            case SqlServer:
                generateMssqlConf(rds);
                break;
            case PostgreSQL:
                generatePostgreSqlConf(rds);
                telegrafService.reload();
                break;
            default:
                log.warn("unsupported resource category " + rds.getType());
                break;
        }
    }

    @Override
    public List<Rds> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = mmsRdsService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(Rds.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Rds targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("rds_id", String.valueOf(targetObject.getId()));
        return params;
    }

    @Override
    public Rds parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String rdsId = String.valueOf(alert.getLabels().get("rds_id"));
        return mmsRdsService.detail(Long.parseLong(rdsId));
    }

    @Override
    public void destroy(Rds rds, String catalog) throws Exception {
        // 资源回收后删除配置文件
        File file = new File(new File(storageConfig.getPrometheusPath(), rds.getType().toLowerCase()), "instance_" + rds.getId() + ".json");
        if (file.exists() && !file.delete()) {
            log.error("file delete failed, path:{} " + file.getPath());
        }
        if (ResourceCategory.MySQL.name().equalsIgnoreCase(rds.getType()) || ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
            // 删除实例节点状态监听rule文件
            File ruleFile = new File(storageConfig.getPrometheusRulesPath(), "alert_rule_event_rds_" + rds.getId() + ".yml");
            if (ruleFile.exists() && !ruleFile.delete()) {
                log.error("file delete failed, path:{} " + ruleFile.getPath());
            }
            prometheusService.reload();
        }
        //TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, Rds rds) {
        alarm.setTarget(String.valueOf(rds.getId()));
        alarm.setTargetName(rds.getName());
        alarm.setOwnerId(rds.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Rds object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public GeneralResult list(Pager pager, String catalog, RequestContext context, HttpServletRequest request) {

        GeneralResult result = mmsRdsService.list(pager, request.getHeader(Common.TOKEN));
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<RdsModel> list = data.getJSONArray(Common.ROWS).toJavaList(RdsModel.class);
        if (list.isEmpty()) {
            data.put(Common.ROWS, list);
            return result;
        }
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(catalog, targets, context);
        for (RdsModel model : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(model.getId().toString()))
                    .findAny().isPresent();
            model.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    /**
     * 生成MySQL配置
     *
     * @param rds
     * @throws IOException
     */
    private void generateMySqlConf(Rds rds) throws IOException {
        // 创建prometheus targets文件
        List<JSONObject> list = rds.getNodes().stream().map(node -> {
            Map<String, String> labels = new HashMap<>(16, 0.75F);
            labels.put("rds_id", String.valueOf(rds.getId()));
            labels.put("node_id", String.valueOf(node.getId()));
            labels.put("server", node.getAddress() + ":" + node.getPort());
            JSONObject config = new JSONObject();
            config.put("targets", Arrays.asList(node.getManageIp() + ":9273"));
            config.put("labels", labels);
            return config;
        }).collect(Collectors.toList());
        JSONArray array = new JSONArray();
        array.addAll(list);
        FileUtils.write(new File(new File(storageConfig.getPrometheusPath(), ResourceCategory.MySQL.name().toLowerCase()), "instance_" + rds.getId() + ".json"), array.toJSONString(), "UTF-8");

        // 创建prometheus rule 文件，用于监听服务节点状态是否变化
        JSONObject obj = new JSONObject();
        JSONArray groups = new JSONArray();
        JSONObject group = new JSONObject();
        group.put("name", "mysql node change event");
        JSONArray rules = new JSONArray();
        Map<String, Object> labels = Maps.newHashMap();
        labels.put("alertRuleType", "nodeEvent");
        labels.put("category", ResourceCategory.MySQL.name());
        labels.put("id", rds.getId());
        JSONObject rule = new JSONObject();
        rule.put("alert", "mysql node change event");
        rule.put("expr", "changes((count(mysql_connections{rds_id='" + rds.getId() + "'}) + 1 or absent(count(mysql_connections{rds_id='" + rds.getId() + "'})))[3m:]) > 0");
        rule.put("for", "1s");
        rule.put("labels", labels);
        rule.put("annotations", MapTools.simpleMap("description", "mysql instance(" + rds.getName() + ") node has changed"));
        rules.add(rule);
        group.put("rules", rules);
        groups.add(group);
        obj.put("groups", groups);
        File file = new File(storageConfig.getPrometheusRulesPath(), "alert_rule_event_rds_" + rds.getId() + ".yml");
        prometheusRuleService.generateFile(Lists.newArrayList(obj), Lists.newArrayList(file));
    }

    /**
     * 生成SQLServer配置
     *
     * @param rds
     * @throws IOException
     */
    private void generateMssqlConf(Rds rds) throws Exception {
        // 创建prometheus targets文件
        List<JSONObject> list = rds.getNodes().stream().map(node -> {
            Map<String, String> labels = new HashMap<>(16, 0.75F);
            labels.put("rds_id", String.valueOf(rds.getId()));
            labels.put("node_id", String.valueOf(node.getId()));
            labels.put("hostname", node.getAddress());
            JSONObject config = new JSONObject();
            config.put("targets", Arrays.asList(node.getManageIp() + ":9273"));
            config.put("labels", labels);
            return config;
        }).collect(Collectors.toList());
        JSONArray array = new JSONArray();
        array.addAll(list);
        FileUtils.write(new File(new File(storageConfig.getPrometheusPath(), ResourceCategory.SqlServer.name().toLowerCase()), "instance_" + rds.getId() + ".json"), array.toJSONString(), "UTF-8");

        // 创建prometheus rule 文件，用于监听服务节点状态是否变化
        JSONObject obj = new JSONObject();
        JSONArray groups = new JSONArray();
        JSONObject group = new JSONObject();
        group.put("name", "sqlserver node change event");
        JSONArray rules = new JSONArray();
        Map<String, Object> labels = Maps.newHashMap();
        labels.put("alertRuleType", "nodeEvent");
        labels.put("category", ResourceCategory.SqlServer.name());
        labels.put("id", rds.getId());
        JSONObject rule = new JSONObject();
        rule.put("alert", "sqlserver node change event");
        rule.put("expr", "changes((count(sqlserver_performance_value{counter='User Connections',rds_id='" + rds.getId() + "'}) + 1 or absent(count(sqlserver_performance_value{counter='User Connections',rds_id='" + rds.getId() + "'})))[3m:]) > 0");
        rule.put("for", "1s");
        rule.put("labels", labels);
        rule.put("annotations", MapTools.simpleMap("description", "sqlserver instance(" + rds.getName() + ") node has changed"));
        rules.add(rule);
        group.put("rules", rules);
        groups.add(group);
        obj.put("groups", groups);
        File file = new File(storageConfig.getPrometheusRulesPath(), "alert_rule_event_rds_" + rds.getId() + ".yml");
        prometheusRuleService.generateFile(Lists.newArrayList(obj), Lists.newArrayList(file));
    }

    /**
     * 生成PostgreSQL配置
     *
     * @param rds
     * @throws IOException
     */
    private void generatePostgreSqlConf(Rds rds) throws IOException {
        // 创建prometheus targets文件
        log.info("开始生成prometheus postgresql相关配置文件 {}",JSONObject.toJSONString(rds));
        List<JSONObject> list = rds.getNodes().stream().map(node -> {
            Map<String, String> labels = new HashMap<>(16, 0.75F);
            labels.put("rds_id", String.valueOf(rds.getId()));
            labels.put("node_id", String.valueOf(node.getId()));
            labels.put("hostname", node.getAddress() + ":" + node.getPort());
            JSONObject config = new JSONObject();
            config.put("targets", Arrays.asList(node.getManageIp() + ":9273"));
            config.put("labels", labels);
            return config;
        }).collect(Collectors.toList());
        JSONArray array = new JSONArray();
        array.addAll(list);
        log.info("配置文件数据为:{}",JSONObject.toJSONString(list));
        FileUtils.write(new File(new File(storageConfig.getPrometheusPath(), ResourceCategory.PostgreSQL.name().toLowerCase()), "instance_" + rds.getId() + ".json"), array.toJSONString(), "UTF-8");
        log.info("生成成功");
        // 创建prometheus rule 文件，用于监听服务节点状态是否变化
        JSONObject obj = new JSONObject();
        JSONArray groups = new JSONArray();
        JSONObject group = new JSONObject();
        group.put("name", "postgresql node change event");
        JSONArray rules = new JSONArray();
        Map<String, Object> labels = Maps.newHashMap();
        labels.put("alertRuleType", "nodeEvent");
        labels.put("category", ResourceCategory.PostgreSQL.name());
        labels.put("id", rds.getId());
        JSONObject rule = new JSONObject();
        rule.put("alert", "postgresql node change event");
        rule.put("expr", "changes((count(postgresql_datid{rds_id='" + rds.getId() + "'}) + 1 or absent(count(postgresql_datid{rds_id='" + rds.getId() + "'})))[3m:]) > 0");
        rule.put("for", "1s");
        rule.put("labels", labels);
        rule.put("annotations", MapTools.simpleMap("description", "postgresql instance(" + rds.getName() + ") node has changed"));
        rules.add(rule);
        group.put("rules", rules);
        groups.add(group);
        obj.put("groups", groups);
        File file = new File(storageConfig.getPrometheusRulesPath(), "alert_rule_event_rds_" + rds.getId() + ".yml");
        prometheusRuleService.generateFile(Lists.newArrayList(obj), Lists.newArrayList(file));
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_RDS_MYSQL, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_RDS_SQLSERVER, this);
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_RDS_POSTGRESQL, this);
    }

}
