package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.intf.PrometheusRuleService;
import com.bocloud.cms.intf.RuleGroupService;
import com.bocloud.cms.intf.ServiceService;
import com.bocloud.cms.model.OpenStackServiceModel;
import com.bocloud.cms.repository.*;
import com.bocloud.cms.service.internal.SmsTenantService;
import com.bocloud.cms.service.internal.SmsUserService;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.google.common.collect.Maps;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @author fanzx
 * @date 2021/09/18
 */
@Service
@Slf4j
public class RuleGroupServiceImpl implements RuleGroupService {

    ThreadFactory generateRuleFactory = new ThreadFactoryBuilder().setNameFormat("generate-rule-pool-%d").build();
    private ExecutorService generateThreadPool = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1,
            Runtime.getRuntime().availableProcessors() + 1, 0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), generateRuleFactory);

    @Autowired
    private RuleGroupRepository ruleGroupRepository;
    @Autowired
    private RuleRepository ruleRepository;
    @Autowired
    private ResourceRelationRepository resourceRelationRepository;
    @Autowired
    private MetricRepository metricRepository;
    @Autowired
    private AlarmSendPolicyRepository alarmSendPolicyRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PrometheusRuleService prometheusRuleService;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private SmsTenantService smsTenantService;
    @Autowired
    private ServiceService serviceService;

    /**
     * 添加告警规则功能
     *
     * @param ruleGroup 告警规则对象
     * @param context   当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @Override
    public GeneralResult create(RuleGroup ruleGroup, RequestContext context) {
        try {
            // 获取catalog信息
            if (RequestContext.Catalog.Manager == context.getCatalog()) {
                ruleGroup.setCreatorcatalog(RequestContext.Catalog.User.name());
            } else {
                ruleGroup.setCreatorcatalog(context.getCatalog().name());
            }
            ruleGroup.setCreatorId(context.getTarget());
            ruleGroup.setTenantId(context.getTenant());
            ruleGroup.setGmtCreate(new Date());
            ruleGroup.setEnable(true);
            ruleGroup.setDeleted(false);
            // 判断告警规则名称是否存在
            RuleGroup existRuleGroup = ruleGroupRepository.getByName(null, ruleGroup.getName(), context);
            if (null != existRuleGroup) {
                return new GeneralResult<>(false, "告警规则名称已存在");
            }
            this.ruleGroupRepository.save(ruleGroup);
            // 保存告警规则信息
            ruleGroup.getRules().forEach(rule -> {
                rule.setGroupId(ruleGroup.getId());
                rule.setGmtCreate(new Date());
                rule.setDeleted(false);
                this.ruleRepository.save(rule);
            });
            // 获取到绑定的资源Id列表
            List<String> resourceIds = ruleGroup.getResourceIds();
            List<ResourceRelation> resourceList = new ArrayList<>();
            if (resourceIds == null || resourceIds.size() == 0) {
                ResourceRelation resourceRelation = new ResourceRelation();
                resourceRelation.setGroupId(ruleGroup.getId());
                resourceRelation.setGmtCreate(new Date());
                resourceRelation.setDeleted(false);
                resourceRelation.setTarget(null);
                this.resourceRelationRepository.save(resourceRelation);
            } else {
                Optional.of(resourceIds).orElse(new ArrayList<>()).forEach(resourceId -> {
                    ResourceRelation resourceRelation = new ResourceRelation(ruleGroup.getId(), resourceId);
                    resourceList.add(resourceRelation);
                });
                Optional.of(resourceList).orElse(new ArrayList<>()).forEach(resourceRelation -> {
                    this.resourceRelationRepository.save(resourceRelation);
                });
            }
            //生成prometheus rule
            generateThreadPool.execute(() -> prometheusRuleService.generateRule(context, ruleGroup));
            return new GeneralResult<>(true, "新增告警规则成功");
        } catch (Exception e) {
            log.error("create ruleGroup error!", e);
            return new GeneralResult<>(false, "新增告警规则失败" + e.getMessage());
        }
    }

    /**
     * 告警规则分页列表展示
     *
     * @param pager   分页   模糊搜索params
     * @param context 当前用户上下文 上下文对象
     * @return 告警规则列表
     */
    @Override
    public GeneralResult list(Pager pager, RequestContext context) {
        GridBean gridBean;
        try {
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = this.ruleGroupRepository.count(pager.getParams(), context);
            List<RuleGroup> ruleGroups = this.ruleGroupRepository.list(pager, context);
            Optional.ofNullable(ruleGroups).orElse(new ArrayList<>()).forEach(ruleGroup -> {
                // 通过groupId找到对应的告警规则rule列表
                List<Rule> rules = this.ruleRepository.listByRuleGroupId(ruleGroup.getId());
                List<Rule> ruleList = new ArrayList<>();
                Optional.ofNullable(rules).orElse(new ArrayList<>()).forEach(rule -> {
                    Metric metric = metricRepository.get(Metric.class, rule.getMetricId());
                    rule.setName(metric.getName());
                    rule.setTag(metric.getTag());
                    rule.setUnit(metric.getUnit());
                    ruleList.add(rule);
                });
                ruleGroup.setRules(ruleList);
                if (!ruleGroup.getIsGlobal()) {
                    // 获取到规则绑定的资源列表
                    List<ResourceRelation> resourceRelations = resourceRelationRepository.listByRuleGroupId(ruleGroup.getId());
                    // 遍历 保存绑定的资源列表Id信息
                    List<String> resourceIds = new ArrayList<>();
                    Optional.ofNullable(resourceRelations).orElse(new ArrayList<>()).forEach(resourceRelation -> {
                        resourceIds.add(resourceRelation.getTarget());
                    });
                    ruleGroup.setResourceIds(resourceIds);
                }
            });
            Optional.ofNullable(ruleGroups).orElse(new ArrayList<>()).forEach(ruleGroup -> {
                // 获取catalog信息
                String creatorcatalog = ruleGroup.getCreatorcatalog();
                if (RequestContext.Catalog.Tenant.name().equals(creatorcatalog)) {
                    GeneralResult<Tenant> result = smsTenantService.detail(ruleGroup.getCreatorId());
                    if (result.isSuccess()) {
                        ruleGroup.setCreateName(result.getData().getName());
                    }
                } else {
                    GeneralResult<User> result = smsUserService.detail(ruleGroup.getCreatorId());
                    if (result.isSuccess()) {
                        ruleGroup.setCreateName(result.getData().getName());
                    }
                }
            });
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, ruleGroups);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("query rulegroup list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    /**
     * 删除告警规则（单条）
     *
     * @param id      告警规则id
     * @param context 当前用户上下文 上下文对象
     * @return 布尔值 成功或者失败
     */
    @Override
    public GeneralResult remove(Long id, RequestContext context) {
        String path = RuleGroup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, id);
            Assert.notNull(ruleGroup, " 数据不存在");
            ruleGroup.setDeleted(true);
            this.ruleGroupRepository.update(ruleGroup);
            this.ruleRepository.delete(id);
            this.resourceRelationRepository.delete(id);
            prometheusRuleService.deleteRuleFile(id, true);
            return new GeneralResult<>(true, "告警规则删除成功!");
        } catch (Exception e) {
            log.error("Remove rulegroup error:", e);
            return new GeneralResult<>(false, "告警规则删除失败：" + e.getMessage());
        }
    }

    /**
     * 批量删除告警规则
     *
     * @param ids       告警规则列表
     * @param contextId 用户ID
     * @return 布尔值 成功或者失败
     */
    @Override
    public GeneralResult batchRemove(List<Long> ids, Long contextId) {
        int rightCount = 0;
        int wrongCount = 0;
        try {
            Assert.notEmpty(ids, " 至少选择一个告警规则删除");
            for (Long id : ids) {
                this.ruleGroupRepository.delete(id);
                this.ruleRepository.delete(id);
                this.resourceRelationRepository.delete(id);
                prometheusRuleService.deleteRuleFile(id, true);
                rightCount++;
            }
        } catch (Exception e) {
            log.error("rulegroup batch delete error!", e);
            return new GeneralResult<>(false, "批量删除告警规则失败:" + e.getMessage());
        }
        return new GeneralResult<>(true, "成功删除【" + rightCount + "】个，失败【" + wrongCount + "】个");
    }

    /**
     * 修改告警规则
     *
     * @param id        告警规则id
     * @param ruleGroup 修改后的告警规则
     * @param context   当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @Override
    public GeneralResult modify(Long id, RuleGroup ruleGroup, RequestContext context) {
        String path = RuleGroup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RuleGroup group = ruleGroupRepository.get(RuleGroup.class, id);
            if (!ruleGroup.getName().equals(group.getName())) {
                boolean exists = ruleGroupRepository.existsSameName(ruleGroup.getName(), context);
                if (exists) {
                    return new GeneralResult<>(false, "告警规则名称已存在");
                }
            }
            group.setIsGlobal(ruleGroup.getIsGlobal());
            // 将传入的分发策略ID保存
            group.setPolicyId(ruleGroup.getPolicyId());
            // 获取catalog信息
            group.setMenderCatalog(context.getCatalog().name());
            group.setGmtCreate(new Date());
            group.setEnable(true);
            group.setDeleted(false);
            group.setRemark(ruleGroup.getRemark());
            group.setName(ruleGroup.getName());
            group.setMenderId(context.getTarget());
            ruleGroupRepository.update(group);
            List<Rule> rules = ruleRepository.listByRuleGroupId(ruleGroup.getId());
            List<Long> oldRules = new ArrayList<>();
            List<Long> newRules = new ArrayList<>();
            List<Rule> ruleList = ruleGroup.getRules();
            rules.forEach(rule -> oldRules.add(rule.getId()));
            ruleList.forEach(rule -> newRules.add(rule.getId()));
            Optional.ofNullable(ruleList).orElse(new ArrayList<>()).forEach(rule -> {
                if (rule.getId() == null) {
                    rule.setGroupId(ruleGroup.getId());
                    rule.setGmtCreate(new Date());
                    rule.setDeleted(false);
                    this.ruleRepository.save(rule);
                } else {
                    rule.setDeleted(false);
                    rule.setGroupId(ruleGroup.getId());
                    ruleRepository.update(rule);
                }
            });
            // 获取到的不同元素就是要被删除的元素
            List<Long> different = this.getDifferent(oldRules, newRules);
            Optional.ofNullable(different).orElse(new ArrayList<>()).forEach(ruleId -> {
                Rule rule = ruleRepository.get(Rule.class, ruleId);
                rule.setDeleted(true);
                rule.setGmtModify(new Date());
                ruleRepository.update(rule);
            });
            resourceRelationRepository.deleteByGroupId(id);
            List<String> resourceIds = ruleGroup.getResourceIds();
            List<ResourceRelation> resourceList = new ArrayList<>();
            if (resourceIds == null || resourceIds.size() == 0) {
                ResourceRelation resourceRelation = new ResourceRelation();
                resourceRelation.setGroupId(ruleGroup.getId());
                resourceRelation.setGmtCreate(new Date());
                resourceRelation.setDeleted(false);
                resourceRelation.setTarget(null);
                this.resourceRelationRepository.save(resourceRelation);
            } else {
                Optional.of(resourceIds).orElse(new ArrayList<>()).forEach(resourceId -> {
                    ResourceRelation resourceRelation = new ResourceRelation(ruleGroup.getId(), resourceId);
                    resourceList.add(resourceRelation);
                });
                Optional.of(resourceList).orElse(new ArrayList<>()).forEach(resourceRelation -> {
                    this.resourceRelationRepository.save(resourceRelation);
                });
            }
            generateThreadPool.execute(() -> prometheusRuleService.generateRule(context, ruleGroup));
            return new GeneralResult<>(true, "告警规则修改成功");
        } catch (Exception exception) {
            log.error("modify rulegroup error:", exception);
            return new GeneralResult<>(false, "告警规则修改失败：" + exception.getMessage());
        }
    }

    /**
     * 告警规则详情展示
     *
     * @param id 告警规则id
     * @return 告警规则详情对象
     */
    @Override
    public RuleGroup detail(Long id) {
        RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, id);
        Assert.notNull(ruleGroup, "未查询到任何数据!");
        List<Rule> rules = this.ruleRepository.listByRuleGroupId(ruleGroup.getId());
        List<Rule> ruleList = new ArrayList<>();
        Optional.ofNullable(rules).orElse(new ArrayList<>()).forEach(rule -> {
            Metric metric = metricRepository.getByMetricId(rule.getMetricId());
            rule.setName(metric.getName());
            rule.setTag(metric.getTag());
            rule.setUnit(metric.getUnit());
            ruleList.add(rule);
        });
        ruleGroup.setRules(ruleList);
        // 判断是否是全局告警
        if (ruleGroup.getIsGlobal()) {
            List<OpenStackServiceModel> openStackServiceModels = serviceService.listByGroupId();
            List<OpenStackServiceModel> result = new ArrayList<>();
            result.addAll(openStackServiceModels);
            ruleGroup.setOpenStackServiceModels(result);
        } else {
            List<ResourceRelation> resourceRelations = resourceRelationRepository.listByRuleGroupId(ruleGroup.getId());
            if (resourceRelations != null) {
                List<String> resourceIds = new ArrayList<>();
                // add 如果类型是OpenStackService 则进行特殊显示  2021/9/11 19:05  fanZx
                if ("MONITOR_OPENSTACK_SERVICE".equals(ruleGroup.getResourceType())) {
                    // 获取到resourceRelation表中对应的资源
                    resourceRelations.forEach(resourceRelation -> resourceIds.add(resourceRelation.getTarget()));
                    // 获取到所有的服务信息 转换成为OpenStackService对象
                    List<OpenStackServiceModel> serviceModels = serviceService.listByGroupId();
                    List<OpenStackServiceModel> result = new ArrayList<>();
                    // 取出两个list中不同的元素 也就是表中的数据和所有service信息
                    serviceModels.forEach(serviceModel -> {
                        for (String resourceId : resourceIds) {
                            // 如果id相同 则代表是已关联的资源
                            if (serviceModel.getId().equals(resourceId)) {
                                result.add(serviceModel);
                            }
                        }
                    });
                    ruleGroup.setOpenStackServiceModels(result);
                } else {
                    resourceRelations.forEach(resourceRelation -> resourceIds.add(resourceRelation.getTarget()));
                }
                ruleGroup.setResourceIds(resourceIds);
            }
        }
        if (ruleGroup.getPolicyId() != null) {
            AlarmSendPolicy alarmSendPolicy = alarmSendPolicyRepository.get(AlarmSendPolicy.class, ruleGroup.getPolicyId());
            if (alarmSendPolicy != null) {
                ruleGroup.setPolicyName(alarmSendPolicy.getName());
            }
        }
        return ruleGroup;
    }

    /**
     * 开启/关闭 告警规则
     *
     * @param groupId 告警规则id
     * @param enable  开启状态 默认为true开启 false为关闭 数据库字段为tinyint
     * @param context 当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @Override
    public GeneralResult enabled(Long groupId, Boolean enable, RequestContext context) {
        try {
            // 获取到告警规则ID 和 现在的状态
            Assert.notNull(groupId, "未查询到要修改的数据!");
            Assert.notNull(enable, "未查询到修改状态!");
            RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, groupId);
            ruleGroup.setEnable(enable);
            ruleGroup.setMenderId(context.getTarget());
            ruleGroupRepository.update(ruleGroup);
            //生成告警规则文件或删除文件
            if (enable) {
                List<Rule> rules = ruleRepository.listByRuleGroupId(ruleGroup.getId());
                ruleGroup.setRules(rules);
                if (!ruleGroup.getIsGlobal()) {
                    // 获取到规则绑定的资源列表
                    List<ResourceRelation> resourceRelations = resourceRelationRepository.listByRuleGroupId(ruleGroup.getId());
                    // 遍历 保存绑定的资源列表Id信息
                    List<String> resourceIds = new ArrayList<>();
                    Optional.ofNullable(resourceRelations).orElse(new ArrayList<>()).forEach(resourceRelation -> {
                        resourceIds.add(resourceRelation.getTarget());
                    });
                    ruleGroup.setResourceIds(resourceIds);
                }
                generateThreadPool.execute(() -> prometheusRuleService.generateRule(context, ruleGroup));
            } else {
                prometheusRuleService.deleteRuleFile(ruleGroup.getId(), true);
            }
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("rulegroup enabled failed:", e);
            return new GeneralResult<>(false, "操作失败" + e.getMessage());
        }
    }

    /**
     * 云平台关联告警规则功能
     *
     * @param ruleGroupIds 告警规则列表集合
     * @param resourceId   资源id
     * @param resourceId   资源类型
     * @param context      当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @Override
    public GeneralResult binding(List<Long> ruleGroupIds, String resourceId, String resourceType, RequestContext context) {
        try {
            Assert.notNull(resourceId, "请选择一个资源");
            Optional.of(ruleGroupIds).orElse(new ArrayList<>()).forEach(ruleGroupId -> {
                ResourceRelation relation = resourceRelationRepository.find(ruleGroupId, resourceId);
                if (relation == null) {
                    // 如果查询为空 则代表库中无数据 进行添加操作;
                    ResourceRelation insert = new ResourceRelation(ruleGroupId, resourceId, new Date(), false);
                    resourceRelationRepository.save(insert);
                }
            });
            List<ResourceRelation> resourceRelations = resourceRelationRepository.listByTarget(resourceId);
            ArrayList<Long> groupId = new ArrayList<>();
            resourceRelations.forEach(resourceRelation -> groupId.add(resourceRelation.getGroupId()));
            // 获取两个List的不同元素 库中已有元素和前端传入元素集合
            List<Long> different = this.getDifferent(groupId, ruleGroupIds);
            Optional.ofNullable(different).orElse(new ArrayList<>()).forEach(ruleGroupId -> {
                // 取消关联操作;
                ResourceRelation resourceRelation = resourceRelationRepository.find(ruleGroupId, resourceId);
                resourceRelation.setDeleted(true);
                resourceRelation.setGmtModify(new Date());
                resourceRelationRepository.update(resourceRelation);
            });
            // 21/9/18 11:02 5.6 bugFix fanzx
            // 云平台关联告警规则之后 会将原来的Prometheus.yml进行删除,并同时产生新的yml文件
            // 取消关联操作之后 需要对生成的Prometheus.yml文件进行修改操作
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("resourceType", resourceType), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("isGlobal", 0), Sign.EQ));
            // 调用list接口 获取到指定资源下的GroupList
            GeneralResult result = list(new Pager(null, null, params, null), context);
            List<RuleGroup> groupList = JSONObject.parseObject(result.getData().toString()).getJSONArray(Common.ROWS).toJavaList(RuleGroup.class);
            Optional.ofNullable(groupList).orElse(new ArrayList<>()).forEach(ruleGroup -> {
                // 先根据groupId 找到将原来的yml文件删除
                prometheusRuleService.deleteRuleFile(ruleGroup.getId(), true);
                // 重新生成新的yml文件
                prometheusRuleService.generateRule(context, ruleGroup);
            });
            return new GeneralResult<>(true, "操作成功");
        } catch (Exception e) {
            log.error("rulegroup relation error!", e);
            return new GeneralResult<>(false, "操作失败" + e.getMessage());
        }
    }

    /**
     * 云平台关联告警规则功能 获取规则资源绑定列表
     * 通过resourceId,查询到已经关联的告警规则列表
     *
     * @param resourceId 资源id
     * @param context    当前用户上下文
     * @return 布尔值 成功或者失败
     */
    @Override
    public GeneralResult bindingList(String resourceId, RequestContext context) {
        try {
            Assert.notNull(resourceId, "请选择一个资源");
            List<Long> ruleGroupIds = new ArrayList<>();
            // 通过resourceId获取到RuleGroup集合对象
            List<ResourceRelation> resourceRelation = resourceRelationRepository.listByTarget(resourceId);
            Optional.ofNullable(resourceRelation).orElse(new ArrayList<>()).forEach(relation -> {
                ruleGroupIds.add(relation.getGroupId());
            });
            return new GeneralResult<>(true, ruleGroupIds, "查询成功");
        } catch (Exception e) {
            log.error("find rulegroup error!", e);
            return new GeneralResult<>(false, "查询失败" + e.getMessage());
        }
    }

    /**
     * 获取两个List的不同元素
     *
     * @param list1 集合1
     * @param list2 集合2
     * @return 返回两个集合中不同的元素
     */
    private List<Long> getDifferent(List<Long> list1, List<Long> list2) {
        List<Long> diff = new ArrayList<>();
        for (Long str : list1) {
            if (!list2.contains(str)) {
                diff.add(str);
            }
        }
        return diff;
    }
}


