package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.common.utils.SshdExecutor;
import com.bocloud.cms.intf.TelegrafService;
import com.bocloud.cms.intf.VendorService;
import com.bocloud.cms.service.internal.CmpCloudVendorService;
import com.bocloud.cms.service.internal.CmpServerService;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.registry.RegistryClient;
import com.megatron.framework.registry.ZooKeeperClient;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * 平台Service实现类
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class VendorServiceImpl implements VendorService {

    private final CmpCloudVendorService cmpCloudVendorService;
    private final CmpServerService cmpServerService;
    private final TelegrafService telegrafService;
    private final StorageConfig storageConfig;
    private final ZooKeeperClient zooKeeperClient;
    private final RegistryClient registryClient;

    @Value("${deploy.mode.docker:false}")
    private boolean isDocker;
    @Value("${deploy.mode.k8s:false}")
    private boolean isK8s;
    @Value("${bocloud.exporter.port:6003}")
    private int port;

    @Override
    public GeneralResult open(RequestContext context, List<Long> idsList) {
        try {
            GeneralResult<CloudVendor> vendorGeneralResult = cmpCloudVendorService.detail(idsList.get(0));
            CloudVendor vendor = vendorGeneralResult.getData();
            switch (CloudProvider.valueOf(vendor.getType())) {
                case VMWARE:
                    openVmware(vendor);
                    break;
                case KUBERNETES:
                    openKubernetes(vendor);
                    break;
                case OPENSTACK:
                case KVM:
                    openOpenstackOrKvm(vendor);
                    break;
                case XSKY:
                    openXsky(vendor);
                    break;
                default:
                    GeneralResult result = cmpCloudVendorService.openMonitoring(idsList);
                    Assert.isTrue(result.isSuccess(), result.getMessage());
                    break;
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new GeneralResult<>(false, "平台开启监控失败:" + e.getMessage());
        }
        return new GeneralResult<>(true, "已开启监控");
    }

    @Override
    public GeneralResult close(RequestContext context, List<Long> idsList) {
        try {
            CloudVendor vendor = cmpCloudVendorService.detail(idsList.get(0)).getData();
            GeneralResult result = cmpCloudVendorService.closeMonitoring(idsList);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            switch (CloudProvider.valueOf(vendor.getType())) {
                case VMWARE:
                    closeVmware(vendor);
                    break;
                case KUBERNETES:
                    closeKubernetes(vendor);
                    break;
                case OPENSTACK:
                case KVM:
                    closeOpenstackOrKvm(vendor);
                    break;
                case XSKY:
                    closeXsky(vendor);
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("平台关闭监控失败", e);
            return new GeneralResult<>(false, "平台开启监控失败:" + e.getMessage());
        }
        return new GeneralResult<>(true, "已关闭监控");
    }

    @Override
    public void updateXskyMonitoringInfo(Long id, JSONObject jsonObject) {
        CloudVendor vendor = cmpCloudVendorService.detail(id).getData();
        JSONObject props = new JSONObject();
        if (StringUtils.isNotBlank(vendor.getProps())) {
            props = JSONObject.parseObject(vendor.getProps());
        }
        JSONObject monitor = new JSONObject();
        monitor.put("xsky", jsonObject);
        props.put("monitor", monitor);
        vendor.setProps(props.toJSONString());
        cmpCloudVendorService.update(vendor);
    }

    /**
     * 开启vcenter监控
     * <p>
     * 1.先预先执行一次重新加载服务保证服务可用
     * 2.生成telegraf所需配置文件并执行
     * 3.再次执行重新加载telegraf服务
     * 4.更新数据库状态
     *
     * @param vendor 云平台
     */
    private void openVmware(CloudVendor vendor) throws Exception {
        String telegrafHost = "";
        int telegrafPort = 22;
        String telegrafUser = "";
        String telegrafPassword = "";
        try {
            String data = this.zooKeeperClient.getData(this.registryClient.getService().getSystem());
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                telegrafHost = object.getString("TelegrafHost");
                telegrafPort = object.getIntValue("TelegrafPort");
                telegrafUser = object.getString("TelegrafUser");
                telegrafPassword = object.getString("TelegrafPassword");
                if (StringUtils.isNotBlank(telegrafPassword)) {
                    // 主机密码解密操作
                    telegrafPassword = new DESEncryptor().decrypt(telegrafPassword, "BocloudCMP");
                }
            }
        } catch (Exception e) {
            log.error("GET ZOOKEEPER DATE ERR:{}", ExceptionUtils.getStackTrace(e));
        }
        restartTelegraf(telegrafHost, telegrafPort, telegrafUser, telegrafPassword);
        telegrafService.createConfigFile(vendor);
        restartTelegraf(telegrafHost, telegrafPort, telegrafUser, telegrafPassword);
        GeneralResult result = cmpCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        Assert.isTrue(result.isSuccess(), result.getMessage());
    }

    /**
     * 重启telegraf
     * @param ip 地址
     * @param port 端口
     * @param user 用户
     * @param password 密码
     */
    private void restartTelegraf(String ip, Integer port, String user, String password) throws IOException {
        if (isDocker) {
            new SshdExecutor(ip, port, user, password).execute("sudo docker restart telegraf");
        } else if (isK8s) {
            new SshdExecutor(ip, port, user, password).execute("source /etc/profile && kubectl -n bocloud-cmp scale deploy/telegraf --replicas=0");
            new SshdExecutor(ip, port, user, password).execute("source /etc/profile && kubectl -n bocloud-cmp scale deploy/telegraf --replicas=1");
        } else {
            new SshdExecutor(ip, port, user, password).execute("sudo systemctl reload telegraf.service");
        }
    }

    /**
     * 开启kubernetes监控
     *
     * @param vendor 云平台
     * @throws Exception
     */
    private void openKubernetes(CloudVendor vendor) throws Exception {
        if (!storageConfig.getKubernetesDir().exists()) {
            storageConfig.getKubernetesDir().mkdirs();
        }
        JSONArray array = new JSONArray();
        JSONObject obj = new JSONObject();
        String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
        address = StringUtils.removeStartIgnoreCase(address, "https://");
        address = StringUtils.removeStartIgnoreCase(address, "http://");
        address = address.substring(0, address.indexOf(':') + 1);
        obj.put("targets", Arrays.asList(address + "30909"));
        array.add(obj);
        File configFile = new File(storageConfig.getKubernetesDir(), vendor.getId() + ".json");
        FileUtils.write(configFile, array.toJSONString(), "utf-8", false);
        GeneralResult result = cmpCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        if (result.isFailed()) {
            if (configFile.exists() && !configFile.delete()) {
                log.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
        JSONObject object = JSONObject.parseObject(result.toString());
        List<CloudVendor> list = JSONArray.parseArray(object.getString("data"), CloudVendor.class);
        if (null == list || list.isEmpty()) {
            if (configFile.exists() && !configFile.delete()) {
                log.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
    }


    /**
     * 开启openstack监控
     *
     * @param vendor 云平台
     * @throws Exception
     */
    private void openOpenstackOrKvm(CloudVendor vendor) throws Exception {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorId", vendor.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("exporterInstalled", true), Sign.EQ));
        GeneralResult serversResult = cmpServerService.list(new Pager(1, Integer.MAX_VALUE, false, params, null));
        JSONObject data = (JSONObject) serversResult.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        List<File> files = new ArrayList<>();
        for (Server server : list) {
            Map<String, String> labels = new HashMap<>(16, 0.75F);
            labels.put("host_id", String.valueOf(server.getId()));
            labels.put("hostname", server.getName());
            labels.put("vendor_id", String.valueOf(vendor.getId()));
            JSONObject config = new JSONObject();
            config.put("targets", Arrays.asList(server.getManageIp() + ":" + port));
            config.put("labels", labels);
            JSONArray array = new JSONArray();
            array.add(config);
            File dir;
            if (CloudProvider.OPENSTACK.name().equalsIgnoreCase(vendor.getType())) {
                dir = storageConfig.getOpenstackDir();
            } else if (CloudProvider.KVM.name().equalsIgnoreCase(vendor.getType())) {
                dir = storageConfig.getKvmDir();
            } else {
                throw new RuntimeException("not support vendor type: " + vendor.getType());
            }
            File file = new File(dir, "server_" + server.getId() + ".json");
            FileUtils.write(file, array.toJSONString(), "UTF-8");
            files.add(file);
        }
        GeneralResult result = cmpCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        if (result.isFailed()) {
            files.forEach(file -> {
                if (file.exists() && !file.delete()) {
                    log.error("file delete failed, path:{} " + file.getPath());
                }
            });
            throw new RuntimeException(result.getMessage());
        }
    }

    /**
     * 开启xsky监控
     *
     * @param vendor 云平台
     * @throws Exception
     */
    private void openXsky(CloudVendor vendor) throws Exception {
        if (!storageConfig.getXskyDir().exists()) {
            storageConfig.getXskyDir().mkdirs();
        }
        Assert.isTrue(StringUtils.isNotBlank(vendor.getProps()), "请先配置数据源");
        JSONObject monitor = JSONObject.parseObject(vendor.getProps()).getJSONObject("monitor");
        Assert.notNull(monitor, "请先配置数据源");
        String address = monitor.getJSONObject("xsky").getString("address");
        Assert.isTrue(StringUtils.isNotBlank(address), "请先配置数据源");
        JSONArray array = new JSONArray();
        JSONObject obj = new JSONObject();
        obj.put("targets", Arrays.asList(StringUtils.remove(StringUtils.remove(address, "http://"), "https://")));
        Map<String, String> labels = new HashMap<>(16, 0.75F);
        labels.put("vendorId", String.valueOf(vendor.getId()));
        obj.put("labels", labels);
        array.add(obj);
        File configFile = new File(storageConfig.getXskyDir(), vendor.getId() + ".json");
        FileUtils.write(configFile, array.toJSONString(), "utf-8", false);
        GeneralResult result = cmpCloudVendorService.openMonitoring(Collections.singletonList(vendor.getId()));
        if (result.isFailed()) {
            if (configFile.exists() && !configFile.delete()) {
                log.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
        JSONObject object = JSONObject.parseObject(result.toString());
        List<CloudVendor> list = JSONArray.parseArray(object.getString("data"), CloudVendor.class);
        if (null == list || list.isEmpty()) {
            if (configFile.exists() && !configFile.delete()) {
                log.error("file delete failed, path:{} " + configFile.getPath());
            }
            throw new RuntimeException(result.getMessage());
        }
    }

    /**
     * 关闭vcenter监控
     *
     * @param vendor 云平台
     */
    private void closeVmware(CloudVendor vendor) throws Exception {
        String telegrafHost = "";
        int telegrafPort = 22;
        String telegrafUser = "";
        String telegrafPassword = "";
        try {
            String data = this.zooKeeperClient.getData(this.registryClient.getService().getSystem());
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                telegrafHost = object.getString("TelegrafHost");
                telegrafPort = object.getIntValue("TelegrafPort");
                telegrafUser = object.getString("TelegrafUser");
                telegrafPassword = object.getString("TelegrafPassword");
                if (StringUtils.isNotBlank(telegrafPassword)) {
                    // 主机密码解密操作
                    telegrafPassword = new DESEncryptor().decrypt(telegrafPassword, "BocloudCMP");
                }
            }
        } catch (Exception e) {
            log.error("GET ZOOKEEPER DATE ERR:{}", ExceptionUtils.getStackTrace(e));
        }
        SshdExecutor executor = new SshdExecutor(telegrafHost, telegrafPort, telegrafUser, telegrafPassword);
        executor.execute(isDocker ? "docker restart telegraf" : "sudo systemctl reload telegraf.service");
        File file = new File(this.storageConfig.getTelegrafPath() + vendor.getUuid() + ".conf");
        if (file.exists()) {
            if (!file.delete()) {
                log.error("file delete failed, path:{} " + file.getPath());
            }
            executor = new SshdExecutor(telegrafHost, telegrafPort, telegrafUser, telegrafPassword);
            executor.execute(isDocker ? "docker restart telegraf" : "sudo systemctl reload telegraf.service");
        }
    }

    /**
     * 关闭kubernetes监控
     *
     * @param vendor 云平台
     */
    private void closeKubernetes(CloudVendor vendor) {
        File file = new File(storageConfig.getKubernetesDir(), vendor.getId() + ".json");
        if (file.exists() && !file.delete()) {
            log.error("file delete failed, path:{} " + file.getPath());
        }
    }


    /**
     * 关闭openstack、kvm监控
     *
     * @param vendor 云平台
     */
    @Override
    public void closeOpenstackOrKvm(CloudVendor vendor) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorId", vendor.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("exporterInstalled", true), Sign.EQ));
        GeneralResult serversResult = cmpServerService.list(new Pager(1, Integer.MAX_VALUE, true, params, null));
        JSONObject data = (JSONObject) serversResult.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        Optional.ofNullable(list).orElse(new ArrayList<>()).forEach(server -> {
            File dir;
            if (CloudProvider.OPENSTACK.name().equalsIgnoreCase(vendor.getType())) {
                dir = storageConfig.getOpenstackDir();
            } else if (CloudProvider.KVM.name().equalsIgnoreCase(vendor.getType())) {
                dir = storageConfig.getKvmDir();
            } else {
                throw new RuntimeException("not support vendor type: " + vendor.getType());
            }
            File file = new File(dir, "server_" + server.getId() + ".json");
            if (file.exists() && !file.delete()) {
                log.error("file delete failed, path:{} " + file.getPath());
            }
        });
    }

    @Override
    public void closeXsky(CloudVendor vendor) {
        File file = new File(storageConfig.getXskyDir(), vendor.getId() + ".json");
        if (file.exists() && !file.delete()) {
            log.error("file delete failed, path:{} " + file.getPath());
        }
    }
}
