package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/9/29 15:01
 */
@Component
public class ApsaraStackAdapter  implements VmAdapter  {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    public ApsaraStackAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository resourceAlarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.resourceAlarmStatusRepository = resourceAlarmStatusRepository;
    }

    @Override
    public void list(List<CloudServerModel> list, RequestContext context) {
        // 获取 aliyun 虚拟机 instanceId 列表
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.APSARASTACK.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        // 装换成 instanceId | instanceId
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");

        List<LabelParam> params = Collections.singletonList(new LabelParam("instanceId", instanceId, Operator.RE_EQ));

        String cpuExpr = "round(apsarastack_ecs_CPUUtilization_average{},0.01)";
        String memoryExpr = "round(apsarastack_ecs_memory_usedutilization_average{},0.01)";

        // prometheus 查询 cpu，memory
        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        // 虚拟机 id 列表
        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());

        // 获取Aliyun资源告警状态
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_APSARASTACK_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            // 设置对应的虚拟机 cpu 利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("instanceId").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            // 设置对应的虚拟机 memory 利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("instanceId").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }

    }

}
