package com.bocloud.cms.service.core;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.AlarmLevel;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmNoticeService;
import com.bocloud.cms.intf.AlarmSendPolicyService;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.repository.AlarmRepository;
import com.bocloud.cms.repository.MonitorResourceTypeRepository;
import com.bocloud.cms.repository.RuleGroupRepository;
import com.bocloud.cms.repository.RuleRepository;
import com.bocloud.cms.service.AlarmConfiguration;
import com.bocloud.ims.entity.CloudServer;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 告警分发核心
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年7月11日
 */
@Slf4j
public class AlarmExecutor implements Runnable {

    private Alarm alarm;
    private Object object;
    private Metric metric;
    private String status;
    private AlarmService alarmService;
    private RuleRepository ruleRepository;
    private RuleGroupRepository ruleGroupRepository;
    private AlarmSendPolicyService alarmSendPolicyService;
    private AlarmNoticeService alarmNoticeService;
    private AlarmRepository alarmRepository;
    private LockFactory lockFactory;
    private MonitorResourceTypeRepository monitorResourceTypeRepository;
    private AlarmConfiguration alarmConfiguration;

    public AlarmExecutor(Alarm alarm, Object object, Metric metric, String status, AlarmService alarmService, RuleRepository ruleRepository,
                         RuleGroupRepository ruleGroupRepository, AlarmSendPolicyService alarmSendPolicyService,
                         AlarmNoticeService alarmNoticeService, AlarmRepository alarmRepository, LockFactory lockFactory,
                         MonitorResourceTypeRepository monitorResourceTypeRepository, AlarmConfiguration alarmConfiguration) {
        super();
        this.alarm = alarm;
        this.object = object;
        this.metric = metric;
        this.status = status;
        this.alarmService = alarmService;
        this.ruleRepository = ruleRepository;
        this.ruleGroupRepository = ruleGroupRepository;
        this.alarmSendPolicyService = alarmSendPolicyService;
        this.alarmNoticeService = alarmNoticeService;
        this.alarmRepository = alarmRepository;
        this.lockFactory = lockFactory;
        this.monitorResourceTypeRepository = monitorResourceTypeRepository;
        this.alarmConfiguration = alarmConfiguration;
    }

    @Override
    public void run() {
        // 忽略告警级别不合法的数据
        if (!AlarmLevel.getValuesMap().containsKey(alarm.getLevel())) {
            log.warn("unkown alarm level:{}", alarm.getLevel());
            return;
        }
        String path = AlarmExecutor.class.getSimpleName() + alarm.getName() + alarm.getLevel() + alarm.getRuleId() +
                alarm.getTarget() + alarm.getTenantId() + alarm.getGmtTrigger();
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 判断主表中是否有存在未解决告警
            Alarm mainAlarmInfo = alarmService.queryAlarmMain(alarm.getName(), alarm.getLevel(), alarm.getRuleId(),
                    alarm.getTarget(), alarm.getTenantId(), alarm.getGmtTrigger());
            if (null != mainAlarmInfo) {
                // 已解决的告警消息
                if ("resolved".equals(status)) {
                    if (!"SOLVED".equals(mainAlarmInfo.getStatus())) {
                        alarmService.solve(mainAlarmInfo.getId(), new RequestContext(RequestContext.Catalog.Manager, 1L, 0L, null, null), "系统解决");
                        //系统解决后发送消息
                        alarm.setId(mainAlarmInfo.getId());
                        alarm.setGmtSolve(new Date());
                        alarm.setRemark(mainAlarmInfo.getRemark());
                        alarmRepository.update(alarm, new String[]{"recoveryVal"});
                        Rule rule = ruleRepository.get(Rule.class, alarm.getRuleId());
                        RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, rule.getGroupId());
                        if (null != ruleGroup.getPolicyId()) {
                            GeneralResult policyResult = alarmSendPolicyService.detail(ruleGroup.getPolicyId());
                            if (policyResult.isFailed()) {
                                log.error("Query send policy error,policy id is" + ruleGroup.getPolicyId());
                                return;
                            }
                            // 告警恢复时同时要给角色下的用户发送恢复信息
                            AlarmSendPolicy policy = (AlarmSendPolicy) policyResult.getData();
                            alarmNoticeService.sendNotice(alarm, policy, true);
                            return;
                        }
                    }
                }
            } else {
                if ("resolved".equals(status)) {
                    return;
                }
                if (null == alarm.getTarget() && null == alarm.getType()) {
                    return;
                }
                // 创建新的告警记录
                alarmService.create(alarm);
                // 发送告警给第三方基础告警
                sendAlarmToOther();
                // 发送通知
                Rule rule = ruleRepository.get(Rule.class, alarm.getRuleId());
                RuleGroup ruleGroup = ruleGroupRepository.get(RuleGroup.class, rule.getGroupId());
                if (null != ruleGroup.getPolicyId()) {
                    GeneralResult policyResult = alarmSendPolicyService.detail(ruleGroup.getPolicyId());
                    if (policyResult.isFailed()) {
                        log.error("Query send policy error,policy id is" + ruleGroup.getPolicyId());
                        return;
                    }
                    AlarmSendPolicy policy = (AlarmSendPolicy) policyResult.getData();
                    // 立即发送
                    if (0L == policy.getWaitTime()) {
                        alarmNoticeService.sendNotice(alarm, policy, false);
                        return;
                    }
                }
            }
        } catch (Exception e) {
            log.error("处理告警消息出错", e);
        }
    }

    private void sendAlarmToOther() {
        try {
            if (!MonitorResourceType.MONITOR_CTSTACK_VM.name().equals(alarm.getType())
                    && MonitorResourceType.MONITOR_MANAGEONE_VM.name().equals(alarm.getType())) {
                return;
            }
            com.bocloud.cms.entity.MonitorResourceType monitorResourceType = monitorResourceTypeRepository.getByResourceType(alarm.getType(), null);
            Map<String, Object> map = new HashMap<>();
            map.put("alarmName", alarm.getName());
            CloudServer server = (CloudServer) object;
            map.put("ip", server.getManagerIp());
            map.put("type", metric.getName());
            map.put("typeName", metric.getName());
            map.put("thresholdId", alarm.getRuleId());
            map.put("sourceId", alarm.getTarget());
            map.put("source", alarm.getTargetName());
            map.put("sourceName", server.getName());
            map.put("category", monitorResourceType.getName());
            map.put("uniform", monitorResourceType.getResourceType());
            map.put("uniformName", monitorResourceType.getName());
            map.put("system", "cmp");
            if (AlarmLevel.URGENT.name().equals(alarm.getLevel())) {
                map.put("severity", 1);
            } else if (AlarmLevel.PRIMARY.name().equals(alarm.getLevel())) {
                map.put("severity", 2);
            } else if (AlarmLevel.SECONDARY.name().equals(alarm.getLevel())) {
                map.put("severity", 3);
            } else if (AlarmLevel.REMIND.name().equals(alarm.getLevel())) {
                map.put("severity", 4);
            }
            map.put("content", alarm.getRemark());
            map.put("createTime", alarm.getGmtTrigger().getTime());

            HttpClient httpClient = new HttpClient();
            httpClient.setDataFormat(PostDataFormat.RAW);
            Map<String, Object> header = new HashMap<>();
            String password = alarmConfiguration.getUsername() + ":" + alarmConfiguration.getPassword();
            String authorization = Base64.getEncoder().encodeToString(password.getBytes());
            header.put("Authorization", "Basic " + authorization);
            Result result = httpClient.post(header, map, alarmConfiguration.getUrl());
            if (result.isFailed()) {
                log.info("发送告警消息到失败, alarm: {}", JSONObject.toJSONString(map));
                throw new RuntimeException(result.getMessage());
            }
            log.info(result.getMessage());
            log.info("发送告警消息到成功, alarm: {}", JSONObject.toJSONString(map));
        } catch (Exception e) {
            log.error("发送告警消息出错", e);
            log.error("alarm:" + JSONObject.toJSONString(alarm));
        }
    }

}
