package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.ims.entity.CloudVendor;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ims云平台调用
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Component
@Slf4j
public class CmpCloudVendorService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    public CmpCloudVendorService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * 云平台详情
     *
     * @param id 云平台ID
     * @return 云平台
     */
    public GeneralResult<CloudVendor> detail(Long id) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/vendors/" + id, HttpMethod.GET,
                headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()){
            log.error("查询云平台失败,id: " + id);
            throw new RuntimeException("查询云平台失败");
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data, CloudVendor.class), "success");
    }

    /**
     * 修改平台信息
     *
     * @param vendor 云平台实体
     * @return 是否成功
     */
    public GeneralResult<?> update(CloudVendor vendor) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/vendors/" + vendor.getId(),
                HttpMethod.PUT, headers, JSONObject.toJSONString(vendor));
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();

    }

    /**
     * 开启监控
     *
     * @param ids 平台ID集合
     * @return 操作是否成功
     */
    public GeneralResult<?> openMonitoring(List<Long> ids) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/vendors/alarm?action=open",
                HttpMethod.PATCH, headers, ids);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    /**
     * 关闭监控
     *
     * @param ids 平台ID集合
     * @return 操作是否成功
     */
    public GeneralResult<?> closeMonitoring(List<Long> ids) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/vendors/alarm?action=close",
                HttpMethod.PATCH, headers, ids);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    /**
     * 云平台列表
     *
     * @param pager 分页
     * @return 云平台列表
     */
    public GeneralResult<?> list(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/vendors", HttpMethod.GET,
                headers, pager.toMap());
        return remoteService.invoke();
    }

    /**
     * 根据云平台ID查询平台
     *
     * @param vendorId 云平台ID
     * @return 云平台
     */
    public GeneralResult<?> listVendor(Long vendorId) {
        List<Param> paramList = new ArrayList<>();
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        paramList.add(new Param(MapTools.simpleMap("id", vendorId), Sign.EQ));
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/vendors", HttpMethod.GET, headers,
                new Pager(1, Integer.MAX_VALUE, paramList, null).toMap(false));
        return remoteService.invoke();
    }

}
