package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.Server;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 远程Worker调用任务分发器，用来对worker请求进行参数校验和请求分片调度。
 */
@Slf4j
@RequiredArgsConstructor
@Component
public class WorkerService {

    private static final String URL = "/api/gaia/v1/job/submit";
    private static final Service SERVICE = Service.create(BoCloudService.GAIA.getName());

    private static final String EXCHANGE = "bocloud.direct.exchange";
    private static final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");
    private static final String QUEUE = "queue";
    private static final String TIMEOUT = "timeout";

    @Value("${bocloud.exporter.port:6003}")
    private int port;

    private final ServiceFactory serviceFactory;
    private final RabbitAdmin rabbitAdmin;
    private final CmpServerService cmpServerService;
    private final CmpCloudVendorService cmpCloudVendorService;
    private final StorageConfig storageConfig;
    private final RabbitTemplate rabbitTemplate;

    /**
     * worker任务分发执行
     *
     * @return 结果
     */
    public GeneralResult<?> execute(Map<String, Object> param, Long serverId, RequestContext context) {
        log.info("start to execute remote worker dispatch ...");
        if (serviceFactory.exist(SERVICE).isFailed()) {
            log.error("remote worker does not exist!");
            return new GeneralResult<>(false, "远程Worker不存在");
        }
        Map<String, Object> headers = MapTools.simpleMap("Content-Type", "application/json");
        RemoteService service = serviceFactory.build(SERVICE, URL, HttpMethod.POST, headers, param);
        Assert.isTrue(null != service && service.isExist(), "远程Worker服务不存在");
        service.setDataFormat(PostDataFormat.RAW);
        try {
            // 处理任务执行的结果
            GeneralResult<?> generalResult = handleAsync(service, serverId, context);
            if (generalResult.isFailed()) {
                // 单次任务请求处理失败！
                log.error("Worker任务执行失败！{}", generalResult.getMessage());
                return new GeneralResult<>(false, "任务执行失败：可能所设定时间已经超时");
            }
            log.info("job of execute success!");
            return generalResult;
        } catch (Exception e) {
            log.error("单次请求worker任务执行异常", e);
            return new GeneralResult<>(false, "任务执行失败：可能所设定时间已经超时");
        }
    }

    /**
     * 处理异步任务
     *
     * @param service  RemoteService
     * @param serverId 宿主机ID
     * @param context  当前用户
     * @return 结果
     */
    private GeneralResult<?> handleAsync(RemoteService service, Long serverId, RequestContext context) {
        String callbackQueue = IDFactory.instance().uuid();
        Map<String, Object> params = (Map<String, Object>) service.getParam().getParams();
        params = Optional.ofNullable(params).orElse(new HashMap<>());
        int timeout = 1800;
        params.put(TIMEOUT, timeout);
        params.put(QUEUE, callbackQueue);
        service.getParam().setParams(params);

        Object object = params.get("module");
        if (null == object) {
            log.info("start to async call remote worker [{}] with callback queue is [{}] and timeout is :{}s",
                    service.getUrl(), callbackQueue, timeout);
        } else {
            Map<String, Object> module = (Map<String, Object>) object;
            log.info("start to async call remote worker [{}] with callback queue is [{}] and ros module is [{}] and timeout is :{}s",
                    service.getUrl(), callbackQueue, module.get("name"), timeout);
        }
        params.put(QUEUE, callbackQueue);
        service.getParam().setParams(params);
        // 声明worker结果的存放队列并将其绑定到指定的exchange:bocloud.direct.exchange
        rabbitAdmin.declareQueue(new Queue(callbackQueue, false, false, true, arguments));
        rabbitAdmin.declareBinding(new Binding(callbackQueue, Binding.DestinationType.QUEUE, EXCHANGE, callbackQueue, arguments));
        log.info("request params is {}", JSONObject.toJSON(service.getParam().getParams()));
        GeneralResult<?> invoke = service.invoke();
        if (invoke.isFailed()) {
            log.error("invoke remote worker [{}] failed for : {}", service.getUrl(), invoke.getMessage());
            return new GeneralResult<>(false, invoke.getMessage());
        }
        new Thread(() -> {
            GeneralResult<?> result;
            try {
                // 这里说明请求成功发送到worker上，接下来可以等待结果消息了
                Message message = rabbitAdmin.getRabbitTemplate().receive(callbackQueue, timeout * 60 * 1000);
                if (null == message) {
                    log.error("get callback from remote worker by queue [{}] timeout", callbackQueue);
                    result = new GeneralResult<>(false, "获取任务结果消息超时");
                } else {
                    String content = new String(message.getBody(), StandardCharsets.UTF_8);
                    log.info("get result [{}] from remote worker by queue [{}]", content, callbackQueue);
                    if (log.isDebugEnabled()) {
                        log.debug("callback from remote worker is :\n{}", content);
                    }
                    try {
                        GeneralResult<?> invokeResult = JSONObject.parseObject(content, GeneralResult.class);
                        if (null == invokeResult.getData()) {
                            result = new GeneralResult<>(false, invokeResult.getMessage());
                        } else {
                            result = invokeResult;
                        }
                    } catch (RuntimeException e) {
                        log.error("callback data transfer exception!", e);
                        result = new GeneralResult<>(false, "返回数据格式异常");
                    }
                }
            } catch (Exception e) {
                log.error("等待Worker 【{}】返回任务结果消息异常:", service.getUrl(), e);
                result = new GeneralResult<>(false, e.getMessage());
            } finally {
                rabbitAdmin.deleteQueue(callbackQueue);
            }
            callbackResult(result, serverId, context);
        }).start();
        return new GeneralResult<>(true, "安装任务已下发");
    }

    /**
     * 处理worker返回结果
     *
     * @param result worker返回结果
     */
    private void callbackResult(GeneralResult<?> result, Long serverId, RequestContext context) {
        try {
            if (result.isSuccess()) {
                log.info(result.getMessage());
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("serverId", serverId);
                jsonObject.put("success", result.isSuccess());
                log.info("send message to {}: {}", "exporter.task.result.queue", JSON.toJSONString(jsonObject));
                rabbitTemplate.convertAndSend("exporter.task.result.queue", jsonObject);
                this.createPrometheusTargetsFile(serverId, context);
            } else {
                OperateResult operateResult = new OperateResult(false, "agent安装失败", "exporter.install.fail", null,
                        OperateResult.OperateCategory.SiteMessage, context);
                rabbitTemplate.convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#",
                        operateResult);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 创建prometheus target文件
     *
     * @param serverId 宿主机ID
     * @param context  当前用户
     * @throws IOException 创建文件失败
     */
    private void createPrometheusTargetsFile(Long serverId, RequestContext context) throws IOException {
        GeneralResult<Server> server = cmpServerService.detail(serverId);
        GeneralResult<CloudVendor> vendor = cmpCloudVendorService.detail(server.getData().getVendorId());
        Map<String, String> labels = new HashMap<>(16, 0.75F);
        labels.put("host_id", String.valueOf(serverId));
        labels.put("hostname", server.getData().getName());
        labels.put("vendor_id", String.valueOf(vendor.getData().getId()));
        JSONObject config = new JSONObject();
        config.put("targets", Collections.singletonList(server.getData().getManageIp() + ":" + port));
        config.put("labels", labels);
        JSONArray array = new JSONArray();
        array.add(config);
        File dir;
        if (CloudProvider.OPENSTACK.name().equalsIgnoreCase(vendor.getData().getType())) {
            dir = storageConfig.getOpenstackDir();
        } else if (CloudProvider.KVM.name().equalsIgnoreCase(vendor.getData().getType())) {
            dir = storageConfig.getKvmDir();
        } else {
            throw new RuntimeException("not support vendor type: " + vendor.getData().getType());
        }
        FileUtils.write(new File(dir, "server_" + serverId + ".json"), array.toJSONString(), "UTF-8");

        // 给前端发送消息以改变安装状态
        OperateResult operateResult = new OperateResult(true, "agent安装成功", "exporter.install.success", null,
                OperateResult.OperateCategory.SiteMessage, context);
        rabbitTemplate.convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#",
                operateResult);
    }

}
